/*
Module : YKUSHWrappers.h
Purpose: Defines the interface for a C++ class which encapsulates the YEPKIT YKUSH XS 
         (https://www.yepkit.com/product/300115/YKUSHXS) switchable hub using  the Windows HID APIs.
History: PJN / 30-12-2020 1. Initial release

Copyright (c) 2020 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/


/////////////////////////// Macros / Defines //////////////////////////////////

#ifndef __YKUSHWRAPPERS_H__
#define __YKUSHWRAPPERS_H__

#ifndef CYKUSHWRAPPERS_EXT_CLASS
#define CYKUSHWRAPPERS_EXT_CLASS
#endif //#ifndef CYKUSHWRAPPERS_EXT_CLASS


#pragma comment(lib, "hid.lib")
#pragma comment(lib, "Setupapi.lib")


/////////////////////////// Includes //////////////////////////////////////////

#include "HIDWrappers.h" //If you get a compilation error about this missing header file, then you need to download my HIDWrappers code from http://www.naughter.com/hidwrappers.html

#ifndef _ARRAY_
#pragma message("To avoid this message, please put array in your pre compiled header (normally stdafx.h)")
#include <array>
#endif //#ifndef _ARRAY_


/////////////////////////// Classes ///////////////////////////////////////////

namespace HID
{

class CYKUSHWRAPPERS_EXT_CLASS CYKUSHXSDevice : public HID::CDevice
{
public:
//Constructors / Destructors
  CYKUSHXSDevice() = default;
  CYKUSHXSDevice(_In_ const CYKUSHXSDevice&) = delete;
  CYKUSHXSDevice(_In_ CYKUSHXSDevice&&) = delete;

//Methods
  CYKUSHXSDevice& operator=(_In_ const CYKUSHXSDevice&) = delete;
  CYKUSHXSDevice& operator=(_In_ CYKUSHXSDevice&&) = delete;

  static bool Enumerate(_Out_ StringArray& deviceNames, _In_ DWORD dwFlags = DIGCF_PRESENT | DIGCF_DEVICEINTERFACE, _In_opt_ LPCTSTR pszEnumerator = nullptr, _In_ USHORT idVendor = 0x4D8, _In_ USHORT idProduct = 0xF0CD)
  {
    return EnumerateDevices(idVendor, idProduct, deviceNames, dwFlags, pszEnumerator);
  }

  _Success_(return != false) bool PortDown() noexcept
  {
    if (!WriteCommand(0x01, 0x01))
      return false;
    BYTE byByte1 = 0;
    BYTE byByte2 = 0;
    if (!ReadResponse(byByte1, byByte2))
      return false;
    return (byByte1 == 0x01);
  }

  _Success_(return != false) bool PortUp() noexcept
  {
    if (!WriteCommand(0x011, 0x011))
      return false;
    BYTE byByte1 = 0;
    BYTE byByte2 = 0;
    if (!ReadResponse(byByte1, byByte2))
      return false;
    return (byByte1 == 0x01);
  }

  _Success_(return != false) bool PortStatus(_Out_ BYTE& byStatus) noexcept
  {
    if (!WriteCommand(0x021, 0x021))
      return false;
    BYTE byByte1 = 0;
    if (!ReadResponse(byByte1, byStatus))
      return false;
    return (byByte1 == 0x01);
  }

protected:
  _Success_(return != false) bool WriteCommand(_In_ BYTE byByte1, _In_ BYTE byByte2) noexcept
  {
    std::array<BYTE, 65> outputReport{};
#pragma warning(suppress: 26446)
    outputReport[1] = byByte1;
#pragma warning(suppress: 26446)
    outputReport[2] = byByte2;
    DWORD dwWritten = 0;
#pragma warning(suppress: 26472)
    return (WriteFile(*this, outputReport.data(), static_cast<DWORD>(outputReport.size()), &dwWritten, nullptr) != FALSE);
  }

  _Success_(return != false) bool ReadResponse(_Out_ BYTE& byByte1, _Out_ BYTE& byByte2) noexcept
  {
    std::array<BYTE, 65> inputReport{};
    DWORD dwRead = 0;
#pragma warning(suppress: 26472)
    const bool bSuccess = (ReadFile(*this, inputReport.data(), static_cast<DWORD>(inputReport.size()), &dwRead, nullptr) != FALSE);
    if (bSuccess)
    {
#pragma warning(suppress: 26446)
      byByte1 = inputReport[1];
#pragma warning(suppress: 26446)
      byByte2 = inputReport[2];
    }
    return bSuccess;
  }
};

}; //namespace HID

#endif //#ifndef __YKUSHWRAPPERS_H__
